@@ -1,5 +1,6 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
 // SPDX-FileCopyrightText: 2022 Nheko Contributors
+// SPDX-FileCopyrightText: 2023 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -7,7 +8,7 @@
 
 #include <QAbstractListModel>
 
-class UsersModel : public QAbstractListModel
+class UsersModel final : public QAbstractListModel
 {
 public:
     enum Roles
