@@ -1,5 +1,6 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
 // SPDX-FileCopyrightText: 2022 Nheko Contributors
+// SPDX-FileCopyrightText: 2023 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -31,11 +32,12 @@ EmojiModel::roleNames() const
     static QHash<int, QByteArray> roles;
 
     if (roles.isEmpty()) {
-        roles                                                 = QAbstractListModel::roleNames();
-        roles[static_cast<int>(EmojiModel::Roles::Unicode)]   = QByteArrayLiteral("unicode");
-        roles[static_cast<int>(EmojiModel::Roles::ShortName)] = QByteArrayLiteral("shortName");
-        roles[static_cast<int>(EmojiModel::Roles::Category)]  = QByteArrayLiteral("category");
-        roles[static_cast<int>(EmojiModel::Roles::Emoji)]     = QByteArrayLiteral("emoji");
+        roles                                                   = QAbstractListModel::roleNames();
+        roles[static_cast<int>(EmojiModel::Roles::Unicode)]     = QByteArrayLiteral("unicode");
+        roles[static_cast<int>(EmojiModel::Roles::ShortName)]   = QByteArrayLiteral("shortName");
+        roles[static_cast<int>(EmojiModel::Roles::UnicodeName)] = QByteArrayLiteral("unicodeName");
+        roles[static_cast<int>(EmojiModel::Roles::Category)]    = QByteArrayLiteral("category");
+        roles[static_cast<int>(EmojiModel::Roles::Emoji)]       = QByteArrayLiteral("emoji");
     }
 
     return roles;
@@ -44,7 +46,7 @@ EmojiModel::roleNames() const
 int
 EmojiModel::rowCount(const QModelIndex &parent) const
 {
-    return parent == QModelIndex() ? Provider::emoji.count() : 0;
+    return parent == QModelIndex() ? (int)Provider::emoji.size() : 0;
 }
 
 QVariant
@@ -58,10 +60,14 @@ EmojiModel::data(const QModelIndex &index, int role) const
             return Provider::emoji[index.row()].unicode;
 
         case Qt::ToolTipRole:
+            return Provider::emoji[index.row()].shortName + ", " +
+                   Provider::emoji[index.row()].unicodeName;
+        case CompletionModel::SearchRole2:
+        case static_cast<int>(EmojiModel::Roles::UnicodeName):
+            return Provider::emoji[index.row()].unicodeName;
         case CompletionModel::SearchRole:
         case static_cast<int>(EmojiModel::Roles::ShortName):
             return Provider::emoji[index.row()].shortName;
-
         case static_cast<int>(EmojiModel::Roles::Category):
             return QVariant::fromValue(Provider::emoji[index.row()].category);
 
