@@ -1,5 +1,6 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
 // SPDX-FileCopyrightText: 2022 Nheko Contributors
+// SPDX-FileCopyrightText: 2023 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -43,8 +44,9 @@ EventStore::EventStore(std::string room_id, QObject *)
       this,
       &EventStore::eventFetched,
       this,
-      [this](
-        std::string id, std::string relatedTo, mtx::events::collections::TimelineEvents timeline) {
+      [this](const std::string &id,
+             const std::string &relatedTo,
+             mtx::events::collections::TimelineEvents timeline) {
           cache::client()->storeEvent(room_id_, id, {timeline});
 
           if (!relatedTo.empty()) {
@@ -79,8 +81,8 @@ EventStore::EventStore(std::string room_id, QObject *)
                   emit beginInsertRows(toExternalIdx(newFirst), toExternalIdx(this->first - 1));
                   this->first = newFirst;
                   emit endInsertRows();
-                  emit fetchedMore();
                   emit dataChanged(toExternalIdx(oldFirst), toExternalIdx(oldFirst));
+                  emit fetchedMore();
               } else {
                   auto range = cache::client()->getTimelineRange(room_id_);
 
@@ -547,8 +549,8 @@ EventStore::edits(const std::string &event_id)
               edits.end(),
               [this, c](const mtx::events::collections::TimelineEvents &a,
                         const mtx::events::collections::TimelineEvents &b) {
-                  return c->getArrivalIndex(this->room_id_, mtx::accessors::event_id(a)) <
-                         c->getArrivalIndex(this->room_id_, mtx::accessors::event_id(b));
+                  return c->getEventIndex(this->room_id_, mtx::accessors::event_id(a)) <
+                         c->getEventIndex(this->room_id_, mtx::accessors::event_id(b));
               });
 
     return edits;
@@ -724,10 +726,11 @@ EventStore::decryptEvent(const IdIndex &idx,
         case olm::DecryptionErrorCode::ParsingFailed:
             break;
         case olm::DecryptionErrorCode::ReplayAttack:
-            nhlog::crypto()->critical("Reply attack while decryptiong event {} in room {} from {}!",
-                                      e.event_id,
-                                      room_id_,
-                                      e.sender);
+            nhlog::crypto()->critical(
+              "Replay attack while decryptiong event {} in room {} from {}!",
+              e.event_id,
+              room_id_,
+              e.sender);
             break;
         case olm::DecryptionErrorCode::NoError:
             // unreachable
@@ -897,6 +900,7 @@ EventStore::fetchMore()
     mtx::http::MessagesOpts opts;
     opts.room_id = room_id_;
     opts.from    = cache::client()->previousBatchToken(room_id_);
+    opts.limit   = 80;
 
     nhlog::ui()->debug("Paginating room {}, token {}", opts.room_id, opts.from);
 
