@@ -1,5 +1,6 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
 // SPDX-FileCopyrightText: 2022 Nheko Contributors
+// SPDX-FileCopyrightText: 2023 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -249,7 +250,7 @@ RoomlistModel::data(const QModelIndex &index, int role) const
 }
 
 void
-RoomlistModel::updateReadStatus(const std::map<QString, bool> roomReadStatus_)
+RoomlistModel::updateReadStatus(const std::map<QString, bool> &roomReadStatus_)
 {
     std::vector<int> roomsToUpdate;
     roomsToUpdate.resize(roomReadStatus_.size());
@@ -283,6 +284,14 @@ RoomlistModel::addRoom(const QString &room_id, bool suppressInsertNotification)
         QSharedPointer<TimelineModel> newRoom(new TimelineModel(manager, room_id));
         newRoom->setDecryptDescription(ChatPage::instance()->userSettings()->decryptSidebar());
 
+        connect(this,
+                &RoomlistModel::currentRoomChanged,
+                newRoom.data(),
+                &TimelineModel::updateLastReadId);
+        connect(MainWindow::instance(),
+                &MainWindow::activeChanged,
+                newRoom.data(),
+                &TimelineModel::lastReadIdOnWindowFocus);
         connect(newRoom.data(),
                 &TimelineModel::newEncryptedImage,
                 MainWindow::instance()->imageProvider(),
@@ -383,7 +392,7 @@ RoomlistModel::addRoom(const QString &room_id, bool suppressInsertNotification)
             currentRoomPreview_->roomid() == room_id) {
             currentRoom_ = models.value(room_id);
             currentRoomPreview_.reset();
-            emit currentRoomChanged();
+            emit currentRoomChanged(room_id);
         }
 
         for (auto p : previewsToAdd) {
@@ -644,12 +653,12 @@ RoomlistModel::clear()
     invites.clear();
     roomids.clear();
     currentRoom_ = nullptr;
-    emit currentRoomChanged();
+    emit currentRoomChanged("");
     endResetModel();
 }
 
 void
-RoomlistModel::joinPreview(QString roomid)
+RoomlistModel::joinPreview(const QString &roomid)
 {
     if (previewedRooms.contains(roomid)) {
         ChatPage::instance()->joinRoomVia(
@@ -715,6 +724,13 @@ RoomlistModel::getRoomPreviewById(QString roomid) const
         if (invites.contains(roomid)) {
             i                 = invites.value(roomid);
             preview.isInvite_ = true;
+
+            auto member = cache::client()->getInviteMember(roomid.toStdString(),
+                                                           http::client()->user_id().to_string());
+
+            if (member) {
+                preview.reason_ = QString::fromStdString(member->reason);
+            }
         } else {
             i                 = previewedRooms.value(roomid);
             preview.isInvite_ = false;
@@ -734,7 +750,7 @@ RoomlistModel::getRoomPreviewById(QString roomid) const
 }
 
 void
-RoomlistModel::setCurrentRoom(QString roomid)
+RoomlistModel::setCurrentRoom(const QString &roomid)
 {
     if ((currentRoom_ && currentRoom_->roomId() == roomid) ||
         (currentRoomPreview_ && currentRoomPreview_->roomid() == roomid))
@@ -743,15 +759,19 @@ RoomlistModel::setCurrentRoom(QString roomid)
     if (roomid.isEmpty()) {
         currentRoom_        = nullptr;
         currentRoomPreview_ = {};
-        emit currentRoomChanged();
+        emit currentRoomChanged("");
     }
 
     nhlog::ui()->debug("Trying to switch to: {}", roomid.toStdString());
     if (models.contains(roomid)) {
         currentRoom_ = models.value(roomid);
         currentRoomPreview_.reset();
-        emit currentRoomChanged();
+        emit currentRoomChanged(currentRoom_->roomId());
         nhlog::ui()->debug("Switched to: {}", roomid.toStdString());
+
+        if (currentRoom_->isSpace()) {
+            emit spaceSelected(roomid);
+        }
     } else if (invites.contains(roomid) || previewedRooms.contains(roomid)) {
         currentRoom_ = nullptr;
         std::optional<RoomInfo> i;
@@ -761,6 +781,13 @@ RoomlistModel::setCurrentRoom(QString roomid)
         if (invites.contains(roomid)) {
             i           = invites.value(roomid);
             p.isInvite_ = true;
+
+            auto member = cache::client()->getInviteMember(roomid.toStdString(),
+                                                           http::client()->user_id().to_string());
+
+            if (member) {
+                p.reason_ = QString::fromStdString(member->reason);
+            }
         } else {
             i           = previewedRooms.value(roomid);
             p.isInvite_ = false;
@@ -781,16 +808,16 @@ RoomlistModel::setCurrentRoom(QString roomid)
                                currentRoomPreview_->roomid_.toStdString());
         }
 
-        emit currentRoomChanged();
+        emit currentRoomChanged("");
     }
 }
 
 namespace {
 enum NotificationImportance : short
 {
-    ImportanceDisabled = -3,
-    NoPreview          = -2,
-    Preview            = -1,
+    NoPreview          = -3,
+    Preview            = -2,
+    ImportanceDisabled = -1,
     AllEventsRead      = 0,
     NewMessage         = 1,
     NewMentions        = 2,
@@ -1070,7 +1097,7 @@ FilteredRoomlistModel::filterAcceptsRow(int sourceRow, const QModelIndex &) cons
 }
 
 void
-FilteredRoomlistModel::toggleTag(QString roomid, QString tag, bool on)
+FilteredRoomlistModel::toggleTag(const QString &roomid, const QString &tag, bool on)
 {
     if (on) {
         http::client()->put_tag(
