@@ -1,5 +1,6 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
 // SPDX-FileCopyrightText: 2022 Nheko Contributors
+// SPDX-FileCopyrightText: 2023 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -31,6 +32,7 @@ class RoomPreview
     Q_PROPERTY(QString roomName READ roomName CONSTANT)
     Q_PROPERTY(QString roomTopic READ roomTopic CONSTANT)
     Q_PROPERTY(QString roomAvatarUrl READ roomAvatarUrl CONSTANT)
+    Q_PROPERTY(QString reason READ reason CONSTANT)
     Q_PROPERTY(bool isInvite READ isInvite CONSTANT)
 
 public:
@@ -40,13 +42,14 @@ public:
     QString roomName() const { return roomName_; }
     QString roomTopic() const { return roomTopic_; }
     QString roomAvatarUrl() const { return roomAvatarUrl_; }
+    QString reason() const { return reason_; }
     bool isInvite() const { return isInvite_; }
 
-    QString roomid_, roomName_, roomAvatarUrl_, roomTopic_;
+    QString roomid_, roomName_, roomAvatarUrl_, roomTopic_, reason_;
     bool isInvite_ = false;
 };
 
-class RoomlistModel : public QAbstractListModel
+class RoomlistModel final : public QAbstractListModel
 {
     Q_OBJECT
     Q_PROPERTY(
@@ -96,7 +99,7 @@ public slots:
     void initializeRooms();
     void sync(const mtx::responses::Sync &sync_);
     void clear();
-    int roomidToIndex(QString roomid)
+    int roomidToIndex(const QString &roomid)
     {
         for (int i = 0; i < (int)roomids.size(); i++) {
             if (roomids[i] == roomid)
@@ -105,27 +108,28 @@ public slots:
 
         return -1;
     }
-    void joinPreview(QString roomid);
+    void joinPreview(const QString &roomid);
     void acceptInvite(QString roomid);
     void declineInvite(QString roomid);
     void leave(QString roomid, QString reason = "");
     TimelineModel *currentRoom() const { return currentRoom_.get(); }
     RoomPreview currentRoomPreview() const { return currentRoomPreview_.value_or(RoomPreview{}); }
-    void setCurrentRoom(QString roomid);
+    void setCurrentRoom(const QString &roomid);
     void resetCurrentRoom()
     {
         currentRoom_ = nullptr;
         currentRoomPreview_.reset();
-        emit currentRoomChanged();
+        emit currentRoomChanged("");
     }
 
 private slots:
-    void updateReadStatus(const std::map<QString, bool> roomReadStatus_);
+    void updateReadStatus(const std::map<QString, bool> &roomReadStatus_);
 
 signals:
     void totalUnreadMessageCountUpdated(int unreadMessages);
-    void currentRoomChanged();
+    void currentRoomChanged(QString currentRoomId);
     void fetchedPreview(QString roomid, RoomInfo info);
+    void spaceSelected(QString roomId);
 
 private:
     void addRoom(const QString &room_id, bool suppressInsertNotification = false);
@@ -152,7 +156,7 @@ private:
     friend class FilteredRoomlistModel;
 };
 
-class FilteredRoomlistModel : public QSortFilterProxyModel
+class FilteredRoomlistModel final : public QSortFilterProxyModel
 {
     Q_OBJECT
     Q_PROPERTY(
@@ -173,7 +177,7 @@ public slots:
     void acceptInvite(QString roomid) { roomlistmodel->acceptInvite(roomid); }
     void declineInvite(QString roomid) { roomlistmodel->declineInvite(roomid); }
     void leave(QString roomid, QString reason = "") { roomlistmodel->leave(roomid, reason); }
-    void toggleTag(QString roomid, QString tag, bool on);
+    void toggleTag(const QString &roomid, const QString &tag, bool on);
     void copyLink(QString roomid);
 
     TimelineModel *currentRoom() const { return roomlistmodel->currentRoom(); }
@@ -218,7 +222,7 @@ public slots:
     void updateHiddenTagsAndSpaces();
 
 signals:
-    void currentRoomChanged();
+    void currentRoomChanged(QString currentRoomId);
 
 private:
     short int calculateImportance(const QModelIndex &idx) const;
