@@ -1,5 +1,6 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
 // SPDX-FileCopyrightText: 2022 Nheko Contributors
+// SPDX-FileCopyrightText: 2023 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -10,8 +11,10 @@
 #include <QStandardPaths>
 #include <QString>
 
+#include "Cache.h"
 #include "ChatPage.h"
 #include "CombinedImagePackModel.h"
+#include "CommandCompleter.h"
 #include "CompletionProxyModel.h"
 #include "EventAccessors.h"
 #include "ImagePackListModel.h"
@@ -149,6 +152,9 @@ TimelineViewManager::TimelineViewManager(CallManager *, ChatPage *parent)
         isConnected_ = true;
         emit isConnectedChanged(true);
     });
+    connect(rooms_, &RoomlistModel::spaceSelected, communities_, [this](QString roomId) {
+        communities_->setCurrentTagId("space:" + roomId);
+    });
 }
 
 void
@@ -246,8 +252,8 @@ TimelineViewManager::escapeEmoji(QString str) const
 
 void
 TimelineViewManager::openImageOverlay(TimelineModel *room,
-                                      QString mxcUrl,
-                                      QString eventId,
+                                      const QString &mxcUrl,
+                                      const QString &eventId,
                                       double originalWidth,
                                       double proportionalHeight)
 {
@@ -377,6 +383,28 @@ TimelineViewManager::queueCallMessage(const QString &roomid,
         room->sendMessageEvent(callHangUp, mtx::events::EventType::CallHangUp);
 }
 
+void
+TimelineViewManager::queueCallMessage(const QString &roomid,
+                                      const mtx::events::voip::CallSelectAnswer &callSelectAnswer)
+{
+    if (auto room = rooms_->getRoomById(roomid))
+        room->sendMessageEvent(callSelectAnswer, mtx::events::EventType::CallSelectAnswer);
+}
+void
+TimelineViewManager::queueCallMessage(const QString &roomid,
+                                      const mtx::events::voip::CallReject &callReject)
+{
+    if (auto room = rooms_->getRoomById(roomid))
+        room->sendMessageEvent(callReject, mtx::events::EventType::CallReject);
+}
+void
+TimelineViewManager::queueCallMessage(const QString &roomid,
+                                      const mtx::events::voip::CallNegotiate &callNegotiate)
+{
+    if (auto room = rooms_->getRoomById(roomid))
+        room->sendMessageEvent(callNegotiate, mtx::events::EventType::CallNegotiate);
+}
+
 void
 TimelineViewManager::focusMessageInput()
 {
@@ -384,7 +412,7 @@ TimelineViewManager::focusMessageInput()
 }
 
 QObject *
-TimelineViewManager::completerFor(QString completerName, QString roomId)
+TimelineViewManager::completerFor(const QString &completerName, const QString &roomId)
 {
     if (completerName == QLatin1String("user")) {
         auto userModel = new UsersModel(roomId.toStdString());
@@ -421,6 +449,11 @@ TimelineViewManager::completerFor(QString completerName, QString roomId)
         auto proxy        = new CompletionProxyModel(stickerModel);
         stickerModel->setParent(proxy);
         return proxy;
+    } else if (completerName == QLatin1String("command")) {
+        auto commandCompleter = new CommandCompleter();
+        auto proxy            = new CompletionProxyModel(commandCompleter);
+        commandCompleter->setParent(proxy);
+        return proxy;
     }
     return nullptr;
 }
@@ -433,7 +466,7 @@ TimelineViewManager::forwardMessageToRoom(mtx::events::collections::TimelineEven
     auto content                                             = mtx::accessors::url(*e);
     std::optional<mtx::crypto::EncryptedFile> encryptionInfo = mtx::accessors::file(*e);
 
-    if (encryptionInfo) {
+    if (encryptionInfo && !cache::isRoomEncrypted(roomId.toStdString())) {
         http::client()->download(
           content,
           [this, roomId, e, encryptionInfo](const std::string &res,
