@@ -1,5 +1,6 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
 // SPDX-FileCopyrightText: 2022 Nheko Contributors
+// SPDX-FileCopyrightText: 2023 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -16,7 +17,7 @@
 #include "Theme.h"
 #include "UserProfile.h"
 
-class Nheko : public QObject
+class Nheko final : public QObject
 {
     Q_OBJECT
 
@@ -57,9 +58,9 @@ public:
     Q_INVOKABLE void showUserSettingsPage() const;
     Q_INVOKABLE void logout() const;
     Q_INVOKABLE void createRoom(bool space,
-                                QString name,
-                                QString topic,
-                                QString aliasLocalpart,
+                                const QString &name,
+                                const QString &topic,
+                                const QString &aliasLocalpart,
                                 bool isEncrypted,
                                 int preset);
     Q_INVOKABLE PowerlevelEditingModels *editPowerlevels(QString room_id_) const
