@@ -1,5 +1,6 @@
 // SPDX-FileCopyrightText: 2021 Nheko Contributors
 // SPDX-FileCopyrightText: 2022 Nheko Contributors
+// SPDX-FileCopyrightText: 2023 Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -13,7 +14,7 @@
 
 typedef struct _GstDevice GstDevice;
 
-class CallDevices : public QObject
+class CallDevices final : public QObject
 {
     Q_OBJECT
 
@@ -43,6 +44,6 @@ private:
     GstDevice *videoDevice(std::pair<int, int> &resolution, std::pair<int, int> &frameRate) const;
 
 public:
-    CallDevices(CallDevices const &) = delete;
+    CallDevices(CallDevices const &)    = delete;
     void operator=(CallDevices const &) = delete;
 };
